(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 30-Dec-2003                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: Billboard.cpp
//
// Desc: Example code showing how to do billboarding. The sample uses
//       billboarding to draw some trees.
//
//       Note: This implementation is for billboards that are fixed to rotate
//       about the Y-axis, which is good for things like trees. For
//       unconstrained billboards, like explosions in a flight sim, the
//       technique is the same, but the the billboards are positioned slightly
//       differently. Try using the inverse of the view matrix, TL-vertices, or
//       some other technique.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit BillboardUnit;

{$I DirectX.inc}

interface

uses
  Windows, Messages, SysUtils,
  Direct3D9, D3DX9, D3DApp, D3DEnumeration, D3DSettings,
  D3DFile, D3DFont, D3DUtil, DXUtil;

const
  NUM_TREES = 500;

// Need global access to the eye direction used by the callback to sort trees
var
  g_vDir: TD3DXVector3;

// Custom vertex type for the trees
type
  PTreeVertex = ^TTreeVertex;
  TTreeVertex = packed record
    p: TD3DXVector3;    // Vertex position
    color: DWORD;       // Vertex color
    tu, tv: Single;     // Vertex texture coordinates
  end;
  PATreeVertex = ^ATreeVertex;
  ATreeVertex = array[0..0] of TTreeVertex;

//-----------------------------------------------------------------------------
// Name: Tree
// Desc: Simple structure to hold data for rendering a tree
//-----------------------------------------------------------------------------
type
  PTree = ^Tree;
  Tree = record
    v: array[0..3] of TTreeVertex; // Four corners of billboard quad
    vPos: TD3DXVector3;            // Origin of tree
    dwTreeTexture: DWORD;          // Which texture map to use
    dwOffset: DWORD;               // Offset into vertex buffer of tree's vertices
  end;

const
  TTreeVertex_FVF = (D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1);

  // Tree textures to use
  g_strTreeTextures: array[0..2] of PChar = (
    'Tree02S.dds',
    'Tree35S.dds',
    'Tree01S.dds'
  );

  NUMTREETEXTURES = 3;

type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Application class. The base class (CD3DApplication) provides the
  //       generic functionality needed in all Direct3D samples. CMyD3DApplication
  //       adds functionality specific to this sample program.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
    m_pFont: CD3DFont;              // Font for drawing text
    m_pTerrain: CD3DMesh;           // Terrain object
    m_pSkyBox: CD3DMesh;            // Skybox background object

    m_pTreeVB: IDirect3DVertexBuffer9;  // Vertex buffer for rendering a tree
    m_pTreeTextures: array[0..NUMTREETEXTURES-1] of IDirect3DTexture9; // Tree images
    m_matBillboardMatrix: TD3DXMatrix; // Used for billboard orientation
    m_Trees: array[0..NUM_TREES-1] of Tree;   // Array of tree info

    m_vEyePt: TD3DXVector3;             // Camera position

    function IsTreePositionValid(iTree: DWORD): Boolean;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;
    // function DrawBackground: HRESULT;
    function DrawTrees: HRESULT;
  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
  public
    constructor Create;
  end;

implementation

uses Math;

// Simple function to define "hilliness" for terrain
function HeightField(x,y: Single): Single;
begin
  Result:= 9*(cos(x/20+0.2)*Cos(y/15-0.2)+1.0);
end;


//-----------------------------------------------------------------------------
// Name: CMyD3DApplication.Create
// Desc: Application constructor. Sets attributes for the app.
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
var
  i: DWORD;
begin
  inherited;
  m_strWindowTitle    := 'Billboard: D3D Billboarding Example';
  m_d3dEnumeration.AppUsesDepthBuffer := True;

  m_pFont        := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);
  m_pSkyBox      := CD3DMesh.Create(PChar('D3D Mesh'));
  m_pTerrain     := CD3DMesh.Create(PChar('D3D Mesh'));
  m_pTreeVB      := nil;

  for i:= 0 to NUMTREETEXTURES - 1 do m_pTreeTextures[i] := nil;
end;


//-----------------------------------------------------------------------------
// Name: IsTreePositionValid
// Desc: Verifies that the tree at the given index is sufficiently spaced
//       from the other trees. If trees are placed too closely, one tree
//       can quickly pop in front of the other as the camera angle changes.
//-----------------------------------------------------------------------------
function CMyD3DApplication.IsTreePositionValid(iTree: DWORD): Boolean;
var
  x, z: Single;
  i: Integer;
  fDeltaX, fDeltaZ: Double;
begin
  Result:= False;

  x := m_Trees[iTree].vPos.x;
  z := m_Trees[iTree].vPos.z;

  for i:= 0 to iTree - 1 do
  begin
    fDeltaX := Abs(x - m_Trees[i].vPos.x);
    fDeltaZ := Abs(z - m_Trees[i].vPos.z);

    if (3.0 > Power(fDeltaX, 2) + Power(fDeltaZ, 2)) then Exit;
  end;

  Result:= True;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
const
  RAND_MAX = $7fff;
var
  i: DWORD;
  fTheta, fRadius: Single;
  fWidth, fHeight: Single;
  r, g, b, dwcolor: DWORD;
begin
  Randomize;
  // Initialize the tree data
  for i:= 0 to NUM_TREES-1 do
  begin
    repeat
      // Position the trees randomly
      fTheta  := 2.0*D3DX_PI*Random;
      fRadius := 25.0 + 55.0*Random;
      m_Trees[i].vPos.x := fRadius * Sin(fTheta);
      m_Trees[i].vPos.z := fRadius * Cos(fTheta);
      m_Trees[i].vPos.y := HeightField(m_Trees[i].vPos.x, m_Trees[i].vPos.z);
    until IsTreePositionValid(i);

    // Size the trees randomly
    fWidth  := 1.0 + 0.2 * (random(RAND_MAX)-random(RAND_MAX))/RAND_MAX;
    fHeight := 1.4 + 0.4 * (random(RAND_MAX)-random(RAND_MAX))/RAND_MAX;

    // Each tree is a random color between red and green
    r := (255-190) + trunc((190*(random(RAND_MAX))/RAND_MAX));
    g := (255-190) + trunc((190*(random(RAND_MAX))/RAND_MAX));
    b := 0;
    dwColor := $ff000000 + (r shl 16) + (g shl 8) + (b shl 0);

    m_Trees[i].v[0].p     := D3DXVector3(-fWidth, 0*fHeight, 0.0);
    m_Trees[i].v[0].color := dwColor;
    m_Trees[i].v[0].tu    := 0.0;   m_Trees[i].v[0].tv := 1.0;
    m_Trees[i].v[1].p     := D3DXVector3(-fWidth, 2*fHeight, 0.0);
    m_Trees[i].v[1].color := dwColor;
    m_Trees[i].v[1].tu    := 0.0;   m_Trees[i].v[1].tv := 0.0;
    m_Trees[i].v[2].p     := D3DXVector3( fWidth, 0*fHeight, 0.0);
    m_Trees[i].v[2].color := dwColor;
    m_Trees[i].v[2].tu    := 1.0;   m_Trees[i].v[2].tv := 1.0;
    m_Trees[i].v[3].p     := D3DXVector3( fWidth, 2*fHeight, 0.0);
    m_Trees[i].v[3].color := dwColor;
    m_Trees[i].v[3].tu    := 1.0;   m_Trees[i].v[3].tv := 0.0;

    // Pick a random texture for the tree
    m_Trees[i].dwTreeTexture := Trunc((NUMTREETEXTURES*random(RAND_MAX)) / RAND_MAX);
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: TreeSortCB()
// Desc: Callback function for sorting trees in back-to-front order
//-----------------------------------------------------------------------------
function TreeSortCB(const arg1, arg2: Pointer): Integer;
const
  Epsilon = 1e-5;
var
  p1, p2: PTree;
  d1, d2: Single;
begin
  p1 := arg1;
  p2 := arg2;

  d1 := p1.vPos.x * g_vDir.x + p1.vPos.z * g_vDir.z;
  d2 := p2.vPos.x * g_vDir.x + p2.vPos.z * g_vDir.z;

  if Abs(d1-d2) < Epsilon then Result:= 0
  else if (d1 < d2) then Result:= +1
  else Result:= -1;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
var
  vUpVec: TD3DXVector3;
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  matView: TD3DXMatrix;
  vDir: TD3DXVector3;
begin
  // Get the eye and lookat points from the camera's path
  vUpVec:= D3DXVector3(0.0, 1.0, 0.0);

  vEyePt.x := 30.0 * Cos(0.8*m_fTime);
  vEyePt.z := 30.0 * Sin(0.8*m_fTime);
  vEyePt.y := 4 + HeightField(vEyePt.x, vEyePt.z);

  vLookatPt.x := 30.0 * Cos(0.8 * (m_fTime + 0.5));
  vLookatPt.z := 30.0 * Sin(0.8 * (m_fTime + 0.5));
  vLookatPt.y := vEyePt.y - 1.0;

  // Set the app view matrix for normal viewing
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  m_pd3dDevice.SetTransform(D3DTS_VIEW, matView);

  // Set up a rotation matrix to orient the billboard towards the camera.
  D3DXVec3Subtract(vDir, vLookatPt, vEyePt);
  // vDir := VectorSub(vLookatPt , vEyePt);
  if (vDir.x > 0.0) then
    D3DXMatrixRotationY(m_matBillboardMatrix, -arctan(vDir.z/vDir.x)+D3DX_PI/2)
  else
    D3DXMatrixRotationY(m_matBillboardMatrix, -arctan(vDir.z/vDir.x)-D3DX_PI/2);
  g_vDir := vDir;

  // Sort trees in back-to-front order
  QSort(@m_Trees, NUM_TREES, SizeOf(Tree), TreeSortCB);

  // Store vectors used elsewhere
  m_vEyePt := vEyePt;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DrawTrees()
// Desc:
//-----------------------------------------------------------------------------
function CMyD3DApplication.DrawTrees: HRESULT;
var
  i: DWORD;
  matWorld: TD3DXMatrixA16;
begin
  // Set diffuse blending for alpha set in vertices.
  m_pd3dDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
  m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

  // Enable alpha testing (skips pixels with less than a certain alpha.)
  if (m_d3dCaps.AlphaCmpCaps and D3DPCMPCAPS_GREATEREQUAL) = D3DPCMPCAPS_GREATEREQUAL then
  begin
    m_pd3dDevice.SetRenderState(D3DRS_ALPHATESTENABLE, iTrue);
    m_pd3dDevice.SetRenderState(D3DRS_ALPHAREF,        $08);
    m_pd3dDevice.SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
  end;

  // Loop through and render all trees
  m_pd3dDevice.SetStreamSource(0, m_pTreeVB, 0, SizeOf(TTreeVertex));
  m_pd3dDevice.SetFVF(TTreeVertex_FVF);
  for i:= 0 to NUM_TREES - 1 do
  begin
    // Quick culling for trees behind the camera
    // This calculates the tree position relative to the camera, and
    // projects that vector against the camera's direction vector. A
    // negative dot product indicates a non-visible tree.
    if (0 > ( m_Trees[i].vPos.x - m_vEyePt.x ) * g_vDir.x +
            ( m_Trees[i].vPos.z - m_vEyePt.z ) * g_vDir.z )
    then Break;

    // Set the tree texture
    m_pd3dDevice.SetTexture(0, m_pTreeTextures[m_Trees[i].dwTreeTexture]);

    // Translate the billboard into place
    m_matBillboardMatrix._41 := m_Trees[i].vPos.x;
    m_matBillboardMatrix._42 := m_Trees[i].vPos.y;
    m_matBillboardMatrix._43 := m_Trees[i].vPos.z;
    m_pd3dDevice.SetTransform(D3DTS_WORLD, m_matBillboardMatrix);

    // Render the billboard
    m_pd3dDevice.DrawPrimitive(D3DPT_TRIANGLESTRIP, m_Trees[i].dwOffset, 2);
  end;

  // Restore state
  D3DXMatrixIdentity(matWorld);
  m_pd3dDevice.SetTransform(D3DTS_WORLD, matWorld );
  m_pd3dDevice.SetRenderState(D3DRS_ALPHATESTENABLE,  iFalse);
  m_pd3dDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, iFalse);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  matView, matViewSave: TD3DXMatrixA16;
begin
  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_ZBUFFER, $00000000, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Render the Skybox
    begin
      // Center view matrix for skybox and disable zbuffer
      m_pd3dDevice.GetTransform(D3DTS_VIEW, matViewSave);
      matView := matViewSave;
      matView._41 := 0.0; matView._42 := -3.0; matView._43 := 0.0;
      m_pd3dDevice.SetTransform(D3DTS_VIEW, matView);
      m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, iFalse);
      // Some cards do not disable writing to Z when
      // D3DRS_ZENABLE is FALSE. So do it explicitly
      m_pd3dDevice.SetRenderState(D3DRS_ZWRITEENABLE, iFalse);

      // Render the skybox
      m_pSkyBox.Render(m_pd3dDevice);

      // Restore the render states
      m_pd3dDevice.SetTransform(D3DTS_VIEW, matViewSave);
      m_pd3dDevice.SetRenderState(D3DRS_ZENABLE, iTrue);
      m_pd3dDevice.SetRenderState(D3DRS_ZWRITEENABLE, iTrue);
    end;

    // Draw the terrain
    m_pTerrain.Render(m_pd3dDevice);

    // Draw the trees
    DrawTrees;

    // Output statistics
    m_pFont.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats);
    m_pFont.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats);

    // End the scene.
    m_pd3dDevice.EndScene;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: This creates all device-dependent managed objects, such as managed
//       textures and managed vertex buffers.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
type
  TVertex = packed record
    x, y, z, tu, tv: Single;
  end;
  PAVertex = ^AVertex;
  AVertex = array[0..MaxInt div SizeOf(TVertex)-1] of TVertex;
var
  i: DWORD;
  v: PATreeVertex;
  iTree: Integer;
  dwOffset: DWORD;
  pVB: IDirect3DVertexBuffer9;
  pVertices: PAVertex;
  dwNumVertices: DWORD;
  strPath: array[0..MAX_PATH-1] of Char;
begin
//  Result:= D3DAPPERR_MEDIANOTFOUND;

  // Initialize the font's internal textures
  m_pFont.InitDeviceObjects(m_pd3dDevice);

  // Create the tree textures
  for i:= 0 to NUMTREETEXTURES - 1 do
  begin
    // Get the path to the texture
    Result:= DXUtil_FindMediaFileCb(strPath, SizeOf(strPath), g_strTreeTextures[i]);
    if FAILED(Result) then Exit;

    // Create the texture using D3DX
    Result:= D3DXCreateTextureFromFileEx(m_pd3dDevice, strPath,
               D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN,
               D3DPOOL_MANAGED, D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR,
               D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR, 0, nil, nil, m_pTreeTextures[i]);
    if FAILED(Result) then Exit;
  end;

  // Create a quad for rendering each tree
  Result:= m_pd3dDevice.CreateVertexBuffer(NUM_TREES*4*SizeOf(TTreeVertex),
                                           D3DUSAGE_WRITEONLY, TTreeVertex_FVF,
                                           D3DPOOL_MANAGED, m_pTreeVB, nil);
  if FAILED(Result) then Exit;

  // Copy tree mesh data into vertexbuffer
  m_pTreeVB.Lock(0, 0, Pointer(v), 0);

  dwOffset := 0;
  for iTree := 0 to NUM_TREES-1 do
  begin
    Move(m_Trees[iTree].v, v[dwOffset], 4*SizeOf(TTreeVertex));
    m_Trees[iTree].dwOffset := dwOffset;
    dwOffset := dwOffset+4;
  end;

  m_pTreeVB.Unlock;

  // Load the skybox
  if FAILED(m_pSkyBox.Create_(m_pd3dDevice, PChar('SkyBox2.x')))
  then Exit;

  // Load the terrain
  if FAILED(m_pTerrain.Create_(m_pd3dDevice, PChar('SeaFloor.x')))
  then Exit;

  // Add some "hilliness" to the terrain
  if SUCCEEDED(m_pTerrain.GetSysMemMesh.GetVertexBuffer(pVB)) then
  begin
    dwNumVertices := m_pTerrain.GetSysMemMesh.GetNumVertices;

    pVB.Lock(0, 0, Pointer(pVertices), 0);

    for i:= 0 to dwNumVertices - 1 do
      pVertices[i].y := HeightField(pVertices[i].x, pVertices[i].z);

    pVB.Unlock;
    pVB:= nil;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Restore device-memory objects and state after a device is created or
//       resized.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  matProj: TD3DXMatrixA16;
  fAspect: Single;
begin
  // Restore the device objects for the meshes and fonts
  m_pTerrain.RestoreDeviceObjects(m_pd3dDevice);
  m_pSkyBox.RestoreDeviceObjects(m_pd3dDevice);
  m_pFont.RestoreDeviceObjects;

  // Set the transform matrices (view and world are updated per frame)
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 1.0, 100.0);
  m_pd3dDevice.SetTransform(D3DTS_PROJECTION, matProj);

  // Set up the default texture states
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
  m_pd3dDevice.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSU,  D3DTADDRESS_CLAMP);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_ADDRESSV,  D3DTADDRESS_CLAMP);

  m_pd3dDevice.SetRenderState(D3DRS_DITHERENABLE, iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_ZENABLE,      iTrue);
  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING,     iFalse);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Called when the device-dependent objects are about to be lost.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  m_pTerrain.InvalidateDeviceObjects;
  m_pSkyBox.InvalidateDeviceObjects;
  m_pFont.InvalidateDeviceObjects;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects:HRESULT;
var
  i: DWORD;
begin
  m_pFont.DeleteDeviceObjects;

  m_pTerrain.Destroy_;
  m_pSkyBox.Destroy_;

  for i:= 0 to NUMTREETEXTURES - 1 do
    SAFE_RELEASE(m_pTreeTextures[i]);

  SAFE_RELEASE(m_pTreeVB);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFont);
  SAFE_DELETE(m_pTerrain);
  SAFE_DELETE(m_pSkyBox);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  if (dwBehavior and D3DCREATE_PUREDEVICE) = D3DCREATE_PUREDEVICE
  then Exit; // GetTransform doesn't work on PUREDEVICE

  // Need to support post-pixel processing (for alpha blending)
  if FAILED(m_pD3D.CheckDeviceFormat(pCaps.AdapterOrdinal, pCaps.DeviceType,
      adapterFormat, D3DUSAGE_RENDERTARGET or D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING,
      D3DRTYPE_SURFACE, backBufferFormat)) then Exit;

  Result:= S_OK;

  // This sample uses alpha textures and/or straight alpha. Make sure the
  // device supports them
  if ((pCaps.TextureCaps and D3DPTEXTURECAPS_ALPHAPALETTE) = D3DPTEXTURECAPS_ALPHAPALETTE)
  then Exit;

  if ((pCaps.TextureCaps and D3DPTEXTURECAPS_ALPHA) = D3DPTEXTURECAPS_ALPHA)
  then Exit;

  Result:= E_FAIL;
end;

end.

